@@ -1,9 +1,9 @@
 /*
- *  $Id: dialog.h,v 1.268 2015/01/25 23:53:19 tom Exp $
+ *  $Id: dialog.h,v 1.310 2024/03/07 15:36:26 tom Exp $
  *
  *  dialog.h -- common declarations for all dialog modules
  *
- *  Copyright 2000-2013,2015	Thomas E. Dickey
+ *  Copyright 2000-2023,2024	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -77,24 +77,6 @@
 #endif
 #endif
 
-/* possible conflicts with <term.h> which may be included in <curses.h> */
-#ifdef color_names
-#undef color_names
-#endif
-
-#ifdef buttons
-#undef buttons
-#endif
-
-#ifdef DLG_ENABLE_NLS
-#include <libintl.h>
-#include <langinfo.h>
-#define _(s) dgettext(PACKAGE, s)
-#else
-#undef _
-#define _(s) s
-#endif
-
 #ifndef GCC_PRINTFLIKE
 #define GCC_PRINTFLIKE(fmt,var) /*nothing*/
 #endif
@@ -107,10 +89,6 @@
 #define GCC_UNUSED /*nothing*/
 #endif
 
-#ifndef DLG_HAVE_WGET_WCH
-#undef USE_WIDE_CURSES
-#endif
-
 /*
  * FIXME: a configure check would be useful
  */
@@ -126,14 +104,27 @@
 #define USE_SHADOW TRUE
 #define USE_COLORS TRUE
 
+/*
+ * These allow using the print-formatting code before curses is initialized.
+ */
+#define DLG_COLS  (COLS  ? COLS  : dialog_state.screen_width)
+#define DLG_LINES (LINES ? LINES : dialog_state.screen_height)
+
+/*
+ * Define the usable size of a window, discounting the area needed for shadow.
+ */
 #ifdef DLG_HAVE_COLOR
-#define SCOLS	(COLS - (dialog_state.use_shadow ? 2 : 0))
-#define SLINES	(LINES - (dialog_state.use_shadow ? 1 : 0))
+#define SCOLS	(DLG_COLS  - (dialog_state.use_shadow ? SHADOW_COLS : 0))
+#define SLINES	(DLG_LINES - (dialog_state.use_shadow ? SHADOW_ROWS : 0))
 #else
 #define SCOLS	COLS
 #define SLINES	LINES
 #endif
 
+/*
+ * These are the default values for exit-codes, which can be overridden by
+ * environment variables, e.g., $DIALOG_CANCEL for DLG_EXIT_CANCEL.
+ */
 #define DLG_EXIT_ESC		255
 #define DLG_EXIT_UNKNOWN	-2	/* never return this (internal use) */
 #define DLG_EXIT_ERROR		-1	/* the shell sees this as 255 */
@@ -142,49 +133,25 @@
 #define DLG_EXIT_HELP		2
 #define DLG_EXIT_EXTRA		3
 #define DLG_EXIT_ITEM_HELP	4	/* actually DLG_EXIT_HELP */
+#define DLG_EXIT_TIMEOUT	5
 
-#define DLG_CTRL(n)	((n) & 0x1f)	/* CTRL is preferred, but conflicts */
-
-#define CHR_HELP	DLG_CTRL('E')
-#define CHR_BACKSPACE	DLG_CTRL('H')
-#define CHR_REPAINT	DLG_CTRL('L')
-#define CHR_KILL	DLG_CTRL('U')
-#define CHR_LITERAL	DLG_CTRL('V')
-#define CHR_DELETE	127
-#define CHR_NEXT	DLG_CTRL('N')
-#define CHR_PREVIOUS	DLG_CTRL('P')
-#define CHR_TRACE	DLG_CTRL('T')
-
-#define ESC		27
-#define TAB		DLG_CTRL('I')
-
-#define MARGIN 1
-#define GUTTER 2
-#define SHADOW_ROWS 1
-#define SHADOW_COLS 2
-#define ARROWS_COL  5
-
-#define MAX_LEN 2048
-#define BUF_SIZE (10L*1024)
-
-#undef  MIN
-#define MIN(x,y) ((x) < (y) ? (x) : (y))
-
-#undef  MAX
-#define MAX(x,y) ((x) > (y) ? (x) : (y))
+#define MARGIN 1	/* width of the line drawn around each box */
+#define GUTTER 2	/* minimum columns between name/description in menu */
+#define SHADOW_ROWS 1	/* rows to reserve for window's shadow */
+#define SHADOW_COLS 2	/* columns to reserve for window's shadow */
+#define ARROWS_COL  5	/* distance from left margin to up/down arrows */
 
 #define DEFAULT_SEPARATE_STR "\t"
 #define DEFAULT_ASPECT_RATIO 9
 /* how many spaces is a tab long (default)? */
 #define TAB_LEN 8
 #define WTIMEOUT_VAL        10	/* minimum amount of time needed for curses */
+#define WTIMEOUT_OFF        -1	/* value to disable timeout */
 
 #ifndef A_CHARTEXT
 #define A_CHARTEXT 0xff
 #endif
 
-#define CharOf(ch)  ((ch) & 0xff)
-
 #ifndef ACS_ULCORNER
 #define ACS_ULCORNER '+'
 #endif
@@ -219,84 +186,10 @@
 #define ACS_BLOCK '#'
 #endif
 
-/* these definitions may work for antique versions of curses */
-#ifndef DLG_HAVE_GETBEGYX
-#undef  getbegyx
-#define getbegyx(win,y,x)	(y = (win)?(win)->_begy:ERR, x = (win)?(win)->_begx:ERR)
-#endif
-
-#ifndef DLG_HAVE_GETMAXYX
-#undef  getmaxyx
-#define getmaxyx(win,y,x)	(y = (win)?(win)->_maxy:ERR, x = (win)?(win)->_maxx:ERR)
-#endif
-
-#ifndef DLG_HAVE_GETPARYX
-#undef  getparyx
-#define getparyx(win,y,x)	(y = (win)?(win)->_pary:ERR, x = (win)?(win)->_parx:ERR)
-#endif
-
-#if !defined(DLG_HAVE_WGETPARENT) && defined(DLG_HAVE_WINDOW__PARENT)
-#undef  wgetparent
-#define wgetparent(win)		((win) ? (win)->_parent : 0)
-#endif
-
-#if !defined(DLG_HAVE_WSYNCUP)
-#undef wsyncup
-#define wsyncup(win) /* nothing */
-#endif
-
-#if !defined(DLG_HAVE_WCURSYNCUP)
-#undef wcursyncup
-#define wcursyncup(win) /* nothing */
-#endif
-
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-/* these definitions may be needed for bleeding-edge curses implementations */
-#if !(defined(DLG_HAVE_GETBEGX) && defined(DLG_HAVE_GETBEGY))
-#undef getbegx
-#undef getbegy
-#define getbegx(win) dlg_getbegx(win)
-#define getbegy(win) dlg_getbegy(win)
-extern int dlg_getbegx(WINDOW * /*win*/);
-extern int dlg_getbegy(WINDOW * /*win*/);
-#endif
-
-#if !(defined(DLG_HAVE_GETCURX) && defined(DLG_HAVE_GETCURY))
-#undef getcurx
-#undef getcury
-#define getcurx(win) dlg_getcurx(win)
-#define getcury(win) dlg_getcury(win)
-extern int dlg_getcurx(WINDOW * /*win*/);
-extern int dlg_getcury(WINDOW * /*win*/);
-#endif
-
-#if !(defined(DLG_HAVE_GETMAXX) && defined(DLG_HAVE_GETMAXY))
-#undef getmaxx
-#undef getmaxy
-#define getmaxx(win) dlg_getmaxx(win)
-#define getmaxy(win) dlg_getmaxy(win)
-extern int dlg_getmaxx(WINDOW * /*win*/);
-extern int dlg_getmaxy(WINDOW * /*win*/);
-#endif
-
-#if !(defined(DLG_HAVE_GETPARX) && defined(DLG_HAVE_GETPARY))
-#undef getparx
-#undef getpary
-#define getparx(win) dlg_getparx(win)
-#define getpary(win) dlg_getpary(win)
-extern int dlg_getparx(WINDOW * /*win*/);
-extern int dlg_getpary(WINDOW * /*win*/);
-#endif
-
-#if !(defined(DLG_HAVE_WGETPARENT) && defined(DLG_HAVE_WINDOW__PARENT))
-#undef wgetparent
-#define wgetparent(win) dlg_wgetparent(win)
-extern WINDOW * dlg_wgetparent(WINDOW * /*win*/);
-#endif
-
 /*
  * This is a list of "old" names, which should be helpful in updating
  * applications that use libdialog.  Starting with 2003/11/26, all exported
@@ -415,6 +308,7 @@ typedef struct _dlg_windows {
     struct _dlg_windows *next;
     WINDOW *normal;
     WINDOW *shadow;
+    int getc_timeout;
 } DIALOG_WINDOWS;
 
 /*
@@ -449,6 +343,19 @@ typedef struct {
     bool finish_string;		/* caching optimization for gauge */
     /* 1.2-20150125 */
     bool plain_buttons;		/* true to suppress button-label highlight */
+    /* 1.3-20180610 */
+    bool text_only;		/* option "--print-text-only", etc. */
+    int text_height;
+    int text_width;
+    /* 1.3-20190211 */
+    int screen_height;
+    int screen_width;
+#ifdef KEY_RESIZE
+    /* 1.3-20190724 */
+    bool had_resize;		/* ERR may follow KEY_RESIZE when polling */
+#endif
+    /* 1.3-20240307 */
+    int color_mode;		/* use colors in textbox, etc? */
 } DIALOG_STATE;
 
 extern DIALOG_STATE dialog_state;
@@ -528,6 +435,19 @@ typedef struct {
     bool last_key;		/* option "--last-key" */
     /* 1.2-20130902 */
     bool help_tags;		/* option "--help-tags" */
+    /* 1.3-20160126 */
+    char *week_start;		/* option "--week-start" */
+    /* 1.3-20160206 */
+    bool iso_week;		/* option "--iso-week" */
+    /* 1.3-20170131 */
+    bool reorder;		/* option "--reorder" */
+    /* 1.3-20201117 */
+    int pause_secs;		/* used by pause widget */
+    /* 1.3-20201126 */
+    bool erase_on_exit;		/* option "--erase-on-exit" */
+    bool cursor_off_label;	/* option "--cursor-off-label" */
+    /* 1.3-20210117 */
+    bool no_hot_list;		/* option "--no-hot-list" */
 } DIALOG_VARS;
 
 #define USE_ITEM_HELP(s)        (dialog_vars.item_help && (s) != 0)
@@ -555,24 +475,20 @@ extern DIALOG_VARS dialog_vars;
 #define chtype long
 #endif
 
-#define UCH(ch)			((unsigned char)(ch))
-
-#define assert_ptr(ptr,msg) if ((ptr) == 0) dlg_exiterr("cannot allocate memory in " msg)
-
-#define dlg_malloc(t,n)    (t *) malloc((size_t)(n) * sizeof(t))
-#define dlg_calloc(t,n)    (t *) calloc((size_t)(n), sizeof(t))
-#define dlg_realloc(t,n,p) (t *) realloc((p), (n) * sizeof(t))
-
 /*
  * Table for attribute- and color-values.
  */
 typedef struct {
-    chtype atr;
+    chtype atr;			/* attribute corresponding to fg, bg, etc */
 #ifdef DLG_HAVE_COLOR
-    int fg;
-    int bg;
-    int hilite;
-#endif
+    int fg;			/* foreground color-number */
+    int bg;			/* background color-number */
+    int hilite;			/* true if bold */
+#ifdef DLG_HAVE_RC_FILE2
+    int ul;			/* true if underline */
+    int rv;			/* true if reverse */
+#endif /* DLG_HAVE_RC_FILE2 */
+#endif /* DLG_HAVE_COLOR */
 #ifdef DLG_HAVE_RC_FILE
     const char *name;
     const char *comment;
@@ -713,6 +629,9 @@ extern void dlg_show_string(WINDOW * /*win*/, const char * /*string*/, int /*off
 extern int dlg_dummy_menutext(DIALOG_LISTITEM * /*items*/, int /*current*/, char * /*newtext*/);
 extern int dlg_renamed_menutext(DIALOG_LISTITEM * /*items*/, int /*current*/, char * /*newtext*/);
 
+/* prgbox.c */
+extern FILE * dlg_popen(const char * /*command */, const char * /*type */);
+
 /* rc.c */
 #ifdef DLG_HAVE_RC_FILE
 extern int dlg_parse_rc(void);
@@ -722,6 +641,9 @@ extern void dlg_create_rc(const char * /*filename*/);
 /* treeview.c */
 extern int dlg_treeview(const char * /*title*/, const char * /*cprompt*/, int /*height*/, int /*width*/, int /*list_height*/, int /*item_no*/, DIALOG_LISTITEM * /*items*/, const char * /*states*/, int * /*depths*/, int /*flag*/, int * /*current_item*/);
 
+/* ttysize.c */
+extern int dlg_ttysize(int /* fd */, int * /* height */, int * /* width */);
+
 /* ui_getc.c */
 extern int dlg_getc(WINDOW * /*win*/, int * /*fkey*/);
 extern int dlg_getc_callbacks(int /*ch*/, int /*fkey*/, int * /*result*/);
@@ -734,16 +656,20 @@ extern void dlg_remove_callback(DIALOG_CALLBACK * /*p*/);
 extern void dlg_killall_bg(int *retval);
 
 /* util.c */
+extern DIALOG_WINDOWS * _dlg_find_window(WINDOW * /* win */);
+extern WINDOW * dlg_der_window(WINDOW * /*win*/, int /*height*/, int /*width*/, int /*y*/, int /*x*/);
 extern WINDOW * dlg_new_modal_window(WINDOW * /*parent*/, int /*height*/, int /*width*/, int /*y*/, int /*x*/);
 extern WINDOW * dlg_new_window(int /*height*/, int /*width*/, int /*y*/, int /*x*/);
 extern WINDOW * dlg_sub_window(WINDOW * /*win*/, int /*height*/, int /*width*/, int /*y*/, int /*x*/);
 extern bool dlg_need_separator(void);
+extern char * dlg_getenv_str(const char * /*name*/);
 extern char * dlg_set_result(const char * /*string*/);
 extern char * dlg_strclone(const char * /*cprompt*/);
 extern char * dlg_strempty(void);
 extern chtype dlg_asciibox(chtype /*ch*/);
 extern chtype dlg_boxchar(chtype /*ch*/);
 extern chtype dlg_get_attrs(WINDOW * /*win*/);
+extern const char * dlg_exitcode2s(int /*code*/);
 extern const char * dlg_print_line(WINDOW * /*win*/, chtype * /*attr*/, const char * /*prompt*/, int /*lm*/, int /*rm*/, int * /*x*/);
 extern int dlg_box_x_ordinate(int /*width*/);
 extern int dlg_box_y_ordinate(int /*height*/);
@@ -755,8 +681,11 @@ extern int dlg_default_item(char ** /*items*/, int /*llen*/);
 extern int dlg_default_listitem(DIALOG_LISTITEM * /*items*/);
 extern int dlg_defaultno_button(void);
 extern int dlg_default_button(void);
+extern int dlg_exitname2n(const char * /*name*/);
+extern int dlg_getenv_num(const char * /*name*/, int * /* value */);
 extern int dlg_max_input(int /*max_len*/);
 extern int dlg_print_scrolled(WINDOW * /* win */, const char * /* prompt */, int /* offset */, int /* height */, int /* width */, int /* pauseopt */);
+extern int dlg_set_timeout(WINDOW * /* win */, bool /* will_getc */);
 extern void dlg_add_help_formitem(int * /* result */, char ** /* tag */, DIALOG_FORMITEM * /* item */);
 extern void dlg_add_help_listitem(int * /* result */, char ** /* tag */, DIALOG_LISTITEM * /* item */);
 extern void dlg_add_quoted(char * /*string*/);
@@ -778,13 +707,16 @@ extern void dlg_draw_bottom_box2(WINDOW * /*win*/, chtype /*on_left*/, chtype /*
 extern void dlg_draw_box(WINDOW * /*win*/, int /*y*/, int /*x*/, int /*height*/, int /*width*/, chtype /*boxchar*/, chtype /*borderchar*/);
 extern void dlg_draw_box2(WINDOW * /*win*/, int /*y*/, int /*x*/, int /*height*/, int /*width*/, chtype /*boxchar*/, chtype /*borderchar*/, chtype /*borderchar2*/);
 extern void dlg_draw_title(WINDOW *win, const char *title);
-extern void dlg_exit(int /*code*/) GCC_NORETURN;
+extern GCC_NORETURN void dlg_exit(int /*code*/);
 extern void dlg_item_help(const char * /*txt*/);
+extern void dlg_keep_tite(FILE * /*output */);
 extern void dlg_print_autowrap(WINDOW * /*win*/, const char * /*prompt*/, int /*height*/, int /*width*/);
 extern void dlg_print_listitem(WINDOW * /*win*/, const char * /*text*/, int /*climit*/, bool /*first*/, int /*selected*/);
+extern void dlg_print_nowrap(WINDOW * /*win*/, const char * /*line*/, int /*width*/);
 extern void dlg_print_size(int /*height*/, int /*width*/);
 extern void dlg_print_text(WINDOW * /*win*/, const char * /*txt*/, int /*len*/, chtype * /*attr*/);
 extern void dlg_put_backtitle(void);
+extern void dlg_reset_timeout(WINDOW * /* win */);
 extern void dlg_restore_vars(DIALOG_VARS * /* save */);
 extern void dlg_save_vars(DIALOG_VARS * /* save */);
 extern void dlg_set_focus(WINDOW * /*parent*/, WINDOW * /*win*/);
@@ -793,7 +725,7 @@ extern void dlg_trim_string(char * /*src*/);
 extern void end_dialog(void);
 extern void init_dialog(FILE * /*input*/, FILE * /*output*/);
 
-extern void dlg_exiterr(const char *, ...) GCC_NORETURN GCC_PRINTFLIKE(1,2);
+extern GCC_NORETURN void dlg_exiterr(const char *, ...) GCC_PRINTFLIKE(1,2);
 
 #ifdef DLG_HAVE_COLOR
 extern chtype dlg_color_pair(int /*foreground*/, int /*background*/);
@@ -811,18 +743,28 @@ extern int dlg_strcmp(const char * /*a*/, const char * /*b*/);
 #ifdef DLG_HAVE_DLG_TRACE
 #define DLG_TRACE(params) dlg_trace_msg params
 extern void dlg_trace_msg(const char *fmt, ...) GCC_PRINTFLIKE(1,2);
+extern void dlg_trace_va_msg(const char *fmt, va_list ap);
+#define DLG_TRACE2S(name,value) dlg_trace_2s (name,value)
+#define DLG_TRACE2N(name,value) dlg_trace_2n (name,value)
+extern void dlg_trace_2s(const char * /*name*/, const char * /*value*/);
+extern void dlg_trace_2n(const char * /*name*/, int /*value*/);
 extern void dlg_trace_win(WINDOW * /*win*/);
 extern void dlg_trace_chr(int /*ch*/, int /*fkey*/);
 extern void dlg_trace(const char * /*fname*/);
 #else
 #define DLG_TRACE(params) /* nothing */
+#define DLG_TRACE2S(name,value) /* nothing */
+#define DLG_TRACE2N(name,value) /* nothing */
+#define dlg_trace_va_msg(fmt, ap) /* nothing */
 #define dlg_trace_win(win) /* nothing */
 #define dlg_trace_chr(ch,fkey) /* nothing */
 #define dlg_trace(fname) /* nothing */
 #endif
 
 #ifdef KEY_RESIZE
+extern void _dlg_resize_cleanup(WINDOW * /*win*/);
 extern void dlg_move_window(WINDOW * /*win*/, int /*height*/, int /*width*/, int /*y*/, int /*x*/);
+extern void dlg_will_resize(WINDOW * /*win*/);
 #endif
 
 /*
@@ -842,7 +784,7 @@ typedef struct mseRegion {
 
 #if defined(NCURSES_MOUSE_VERSION)
 
-#define	mouse_open() mousemask(BUTTON1_CLICKED, (mmask_t *) 0)
+#define	mouse_open() mousemask(BUTTON1_PRESSED, (mmask_t *) 0)
 #define	mouse_close() mousemask(0, (mmask_t *) 0)
 
 extern mseRegion * dlg_mouse_mkregion (int /*y*/, int /*x*/, int /*height*/, int /*width*/, int /*code*/);
@@ -891,16 +833,6 @@ extern int dlg_mouse_wgetch_nowait (WINDOW * /*win*/, int * /*fkey*/);
 #define FLAG_CHECK 1
 #define FLAG_RADIO 0
 
-/*
- * This is used only for debugging (FIXME: should have a separate header).
- */
-#ifdef DLG_NO_LEAKS
-extern void _dlg_inputstr_leaks(void);
-#if defined(NCURSES_VERSION) && defined(DLG_HAVE__NC_FREE_AND_EXIT)
-extern void _nc_free_and_exit(int);	/* nc_alloc.h normally not installed */
-#endif
-#endif
-
 #ifdef __cplusplus
 }
 #endif
